import logging
import re

from django.contrib import admin
from django.contrib.admin import AdminSite
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User
from django.db.models import Count
from django.forms import CheckboxSelectMultiple
from django.urls import reverse
from django.utils.html import format_html

from .models import Tenant, TenantUser, TenantUserGroup, TenantUserGroupIPAuth, Publisher, EResource, \
    EResourceLegacyData, EResourceDBData, ResourceTenantData, EResourcePDFData, DBoDGeneralSettings, \
    TenantUserGroupShibbUser
from .utils import read_link_from_dbodmysql, dbodip

logger = logging.getLogger(__name__)


class MyAdminSite(AdminSite):
    index_template = 'dbod/admin/dbodindex.html'
    site_header = 'DBoD 2 - Admin'
    site_title = 'DBoD 2'


def deactivate(modeladmin, request, queryset):
    queryset.update(is_active=False)


deactivate.short_description = "Deaktivieren"


def activate(modeladmin, request, queryset):
    queryset.update(is_active=True)


activate.short_description = "Aktivieren"


class TenantAdmin(admin.ModelAdmin):
    list_display = ('code', 'name', 'is_active')
    list_filter = ('is_active',)
    ordering = ['code']

    actions = [deactivate, activate]


class UserInline(admin.StackedInline):
    model = User


class TenantUserAdmin(admin.ModelAdmin):
    def email(self, obj):
        return obj.user.email

    def readabletenant(self, obj):
        return obj.tenant.name + " (" + obj.tenant.code + ")"

    list_display = ('email', 'readabletenant', 'is_active')
    actions = [deactivate, activate]


class TUGIpInline(admin.TabularInline):
    model = TenantUserGroupIPAuth
    readonly_fields = ['ipstr']


class TUGShibbInline(admin.TabularInline):
    model = TenantUserGroupShibbUser


class TenantUserGroupAdmin(admin.ModelAdmin):
    def readabletenant(self, obj):
        return obj.tenant.name + " (" + obj.tenant.code + ")"

    list_display = ('code', 'name', 'readabletenant', 'type', 'is_active')
    list_filter = ('is_active', 'tenant__code', 'type',)
    ordering = ['code']
    inlines = [TUGIpInline, TUGShibbInline]

    def convert_IPAuth(self, request, queryset):
        dbs = DBoDGeneralSettings()
        if dbs.DEVMODE:
            TenantUserGroupIPAuth.objects.filter(desc__contains='generiert').delete()

        for obj in queryset:
            tug: TenantUserGroup = obj
            tugips = TenantUserGroupIPAuth.objects.filter(tenantusergroup=tug.code)
            for tugip in tugips:
                if tugip.ipstr:
                    addresses = dbodip(tugip.ipstr)
                    if addresses:
                        tugip.is_active = False
                        if not tugip.desc:
                            tugip.desc = tug.code + " IPs"
                        tugip.save()
                        for addr in addresses:
                            ipaddr, network, v6 = addr
                            tugip_new = TenantUserGroupIPAuth(
                                desc=tugip.desc + " generiert",
                                tenantusergroup=tugip.tenantusergroup,
                                ipaddr=ipaddr,
                                network=network,
                                v6=v6
                            )
                            tugip_new.save()

    convert_IPAuth.short_description = 'Konvertiere IP-String zu einer Adresse oder einem Netzwerk'

    actions = [deactivate, activate, convert_IPAuth]


class PublisherAdmin(admin.ModelAdmin):
    list_display = ('code', 'name', 'ressources_count', 'is_active')
    search_fields = ('code', 'name')
    list_filter = ('is_active',)
    change_links = ['ressources_count']
    ordering = ['code']

    actions = [deactivate, activate]

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'tenants':
            kwargs['widget'] = CheckboxSelectMultiple
        return super(PublisherAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def ressources_count(self, obj):
        baseurl = reverse("admin:DBoD_eresource_changelist") + '?publisher__code__exact=' + obj.code
        return format_html('<a href="{}">{}</a>', baseurl, obj.ressources_count)

    ressources_count.short_description = 'Anzahl eRessourcen'

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = queryset.annotate(ressources_count=Count("eresource"))
        return queryset


class ERLegacyInline(admin.StackedInline):
    model = EResourceLegacyData


class ERDBDataInline(admin.StackedInline):
    model = EResourceDBData


class RTenantDataInline(admin.TabularInline):
    model = ResourceTenantData


class RTenantPDFInline(admin.TabularInline):
    model = EResourcePDFData


class EResourceAdmin(admin.ModelAdmin):
    save_as = True
    list_display = ('code', 'name', 'publisher_url', 'restype', 'is_active')
    list_filter = ('is_active', 'restype')
    search_fields = ('code', 'name',)
    inlines = [ERLegacyInline, ERDBDataInline, RTenantPDFInline, RTenantDataInline]
    ordering = ['code']

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'tenantuser':
            kwargs['widget'] = CheckboxSelectMultiple
            if 'object_id' in kwargs['request'].resolver_match.kwargs:
                id = kwargs['request'].resolver_match.kwargs['object_id']
                id = id.replace('_5F', '_')
                res = EResource.objects.get(code=id)
                tenants = res.publisher.tenants.all()
                tenantuser = TenantUserGroup.objects.filter(tenant__code__in=tenants)
                kwargs['queryset'] = tenantuser
        return super(EResourceAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def clean_version(modeladmin, request, queryset):
        for obj in queryset:
            try:
                if obj.eresourcelegacydata.version is not None and obj.eresourcelegacydata.version == '1':
                    obj.eresourcelegacydata.version = None
                    obj.eresourcelegacydata.save()
                    obj.save()
            except EResourceLegacyData.DoesNotExist:
                pass

    clean_version.short_description = 'Version = 1 entfernen'

    def convert_wwwDB(modeladmin, request, queryset):
        for obj in queryset:
            try:
                if obj.eresourcelegacydata.info is not None and re.match(r'\d+', obj.eresourcelegacydata.info):
                    obj.dbisid = obj.eresourcelegacydata.info
                    obj.save()
            except EResourceLegacyData.DoesNotExist:
                pass

            for trd in ResourceTenantData.objects.filter(eresource=obj):
                link, shib = read_link_from_dbodmysql(ressource=obj.code, tenant=trd.tenant.code)
                if link and (link.startswith('http://wwwdb.dbod.de/login?url=') or link.startswith(
                        'https://wwwdb.dbod.de/login?url=')):
                    trd.proxyurl = 'http://wwwdb.dbod.de/login?url='
                    link = link.replace('http://wwwdb.dbod.de/login?url=', '')
                    link = link.replace('https://wwwdb.dbod.de/login?url=', '')
                trd.link = link
                trd.shiburl = shib
                if link:
                    trd.user = None
                    trd.pwd = None
                trd.save()

            obj.save()

    convert_wwwDB.short_description = 'Alte Daten konvertieren'

    actions = [deactivate, activate, clean_version, convert_wwwDB]

    def publisher_url(self, obj):
        url = reverse("admin:DBoD_publisher_change", args=(obj.publisher.code,))
        return format_html('<a href="{}">{}</a>', url, obj.publisher)


# Unregister the provided model admin
admin.site.unregister(User)


# Register out own model admin, based on the default UserAdmin
@admin.register(User)
class CustomUserAdmin(UserAdmin):
    def give_passwd(self, request, queryset):
        for obj in queryset:
            obj.set_password('dboddbod')
            obj.save()

    give_passwd.short_description = 'Bekanntes Passwort setzen'

    actions = ['give_passwd']


admin.site = MyAdminSite()

admin.site.register(Tenant, TenantAdmin)
admin.site.register(TenantUser, TenantUserAdmin)
admin.site.register(TenantUserGroup, TenantUserGroupAdmin)

admin.site.register(Publisher, PublisherAdmin)
admin.site.register(EResource, EResourceAdmin)

admin.site.register(User, CustomUserAdmin)
