import dbsettings
from django.conf import settings
from django.contrib.auth.models import AbstractUser
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models

# Settings
from dbodruntime.models import CitrixWorkerGroup


class DBoDGeneralSettings(dbsettings.Group):
    DEVMODE = dbsettings.BooleanValue(help_text='Aktierung Dev-Modus', default=False)


class DBoDSettings(dbsettings.Group):
    DBPATH = dbsettings.StringValue(help_text='Pfad zu den Datenbank IBF Dateien (überholt!)', default='/out/db')

    ORGPATH = dbsettings.StringValue(help_text='Pfad zu den Organisations IBF Dateien (überholt!)', default='/out/org')

    CONTACTFILE = dbsettings.StringValue(help_text='Datei in der Kontakte stehen (überholt)', default='contact.ibf')

    MASTERFILE = dbsettings.StringValue(help_text='Master Datei', default='/master/master.ibf')


class DBoDFrontend(dbsettings.Group):
    ITEMSPERPAGE = dbsettings.PositiveIntegerValue(help_text='Anzahl Zeilen pro Seite', default=20)
    SHIBLOGINURL = dbsettings.StringValue(help_text='Shibboleth Login', default='/Shibboleth.sso/Login')
    SHIBLOGOUTURL = dbsettings.StringValue(help_text='Shibboleth Logout', default='/Shibboleth.sso/Logout')


class DBoDLegacy(dbsettings.Group):
    MYSQLHOST = dbsettings.StringValue(help_text='Hostname für MySQL', default='localhost')

    MYSQLPORT = dbsettings.PositiveIntegerValue(help_text='Port für MySQL', default=3306)

    MYSQLUSER = dbsettings.StringValue(help_text='Username für MySQL', default='dbodStat')

    MYSQLPASSWD = dbsettings.PasswordValue(help_text='Password für MySQL', )

    MYSQLDB = dbsettings.StringValue(help_text='Database name für MySQL', default='dbod')


dsettings = DBoDGeneralSettings('Generelle Einstellungen') + DBoDSettings(
    'DBoD Einstellungen (IBF Dateien)') + DBoDFrontend('DBoD Frontend Einstellungen') + DBoDLegacy(
    'Einstellungen für die Übernahme von alten Daten')


# Tenant section

class Tenant(models.Model):
    REDI = 'ReDI'
    TYPE_CHOICES = [(REDI, 'ReDI'), ]

    code = models.CharField('Code', max_length=50, null=False, unique=True, primary_key=True,
                            help_text='Eindeutiger Code des Mandanten')
    name = models.CharField('Bezeichnung', max_length=200, null=False, help_text='Name des Mandanten')
    desc = models.CharField('Beschreibung', max_length=200, null=True, blank=True,
                            help_text='Beschreibung des Mandanten')
    tel = models.CharField('Telefonnummer', max_length=50, null=True, blank=True, help_text='Telefonnummer')
    email = models.EmailField('eMail', max_length=200, null=True, blank=True, help_text='eMail des Mandanten')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Aktiv')

    type = models.CharField(max_length=10, choices=TYPE_CHOICES, default=REDI)
    sigel = models.CharField(max_length=10, null=True, blank=True)
    dbis = models.CharField('DBIS Code', max_length=10,
                            help_text='Code zum Anzeigen einer Mandantenspezifischen DBIS-Seite', null=True, blank=True)
    ezb = models.CharField(max_length=10, null=True, blank=True)
    shibb = models.URLField('Shibboleth Adresse', null=True, blank=True, help_text='Shibboleth Adresse ihres IDP')

    def __str__(self):
        return self.code

    class Meta:
        verbose_name = 'Mandant'
        verbose_name_plural = 'Mandanten'
        ordering = ['code']


class TenantUser(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, verbose_name='Systemnutzer', on_delete=models.CASCADE,
                                primary_key=True)
    tel = models.CharField('Telefonnummer', max_length=50, null=True, blank=True, help_text='Telefonnummer')
    tenant = models.ForeignKey(Tenant, verbose_name='Mandant', on_delete=models.CASCADE)
    is_tenantadmin = models.BooleanField('Mandantenadmin', default=False,
                                         help_text='Ist dieser Nutzer Administrator des Mandanten?')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Ist der Nutzer aktiviert?')

    def __str__(self):
        return '{} ({})'.format(self.user.first_name + " " + self.user.last_name, self.tenant.name)

    class Meta:
        verbose_name = 'Mandanten Login'
        verbose_name_plural = 'Mandanten Logins'


class TenantUserGroup(models.Model):
    USER = 'U'
    ADMIN = 'A'
    TEST = 'T'
    TYPE_CHOICES = [(USER, 'user'), (ADMIN, 'admin'), (TEST, 'test'), ]
    code = models.CharField('Code', max_length=50, null=False, unique=True, help_text='Eindeutiger Code',
                            primary_key=True)
    name = models.CharField('Bezeichnung', max_length=200, null=False, help_text='Bezeichnung')
    desc = models.CharField('Beschreibung', max_length=200, null=True, blank=True, help_text='Beschreibung')
    shibb_affiliate = models.CharField('Shibboleth-Gruppe', max_length=200, null=True, blank=True,
                                       help_text='Vollständiger Shibboleth Bezeichner einer Gruppe')
    shibb_nameduser = models.BooleanField('Einzelnutzer', default=False,
                                          help_text='Ist dies eine Gruppe benannter Nutzer?')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Aktiv')

    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE)
    type = models.CharField(max_length=1, choices=TYPE_CHOICES, default=USER)

    def __str__(self):
        return self.code

    class Meta:
        verbose_name = 'Nutzergruppe'
        verbose_name_plural = 'Nutzergruppen'
        ordering = ['code']


class TenantUserGroupIPAuth(models.Model):
    tenantusergroup = models.ForeignKey(TenantUserGroup, verbose_name="Mandant", on_delete=models.CASCADE)
    desc = models.CharField('Beschreibung', max_length=200, null=True, blank=True,
                            help_text='Beschreibung des IP Bereichs')
    ipaddr = models.CharField('IP Adresse', max_length=200, null=True, blank=True,
                              help_text='IP Adresse oder Netzwerk in CDIR')
    ipstr = models.CharField('IP Addresse als String', max_length=100, null=True, blank=True, editable=False)
    network = models.BooleanField('Netzwerk', default=False, null=True,
                                  help_text='Dies ist ein Netzwerk, keine einzelne Adresse')
    v6 = models.BooleanField('IPv6', default=False, null=True, help_text='Dies ist ein IPv6 Netzwerk oder Adresse')
    prio = models.IntegerField('Priorität', default=10, help_text='Aufsteigende sortierung. 0 = Max')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Aktiv')

    class Meta:
        verbose_name = 'Nutzergruppen IP'
        verbose_name_plural = 'Nutzergruppen IPs'


class TenantUserGroupShibbUser(models.Model):
    tenantusergroup = models.ForeignKey(TenantUserGroup, verbose_name="Mandant", on_delete=models.CASCADE)
    eppn = models.CharField('EPPN', max_length=200, help_text='Nutzerkennung Shibboleth')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Aktiv')

    class Meta:
        verbose_name = 'Nutzer Shibboleth'
        verbose_name_plural = 'Nutzer Shibboleth'


# Resources section

class PManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True)


class Publisher(models.Model):
    code = models.CharField(max_length=50, help_text='Eindeutiger Code', unique=True, primary_key=True)
    name = models.CharField(max_length=200, help_text='Bezeichnung, Name')
    description = models.CharField(max_length=200, null=True, blank=True, help_text='Beschreibung')
    tenants = models.ManyToManyField(Tenant)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Aktiv')

    objects = models.Manager()
    dbod_objects = PManager()

    class Meta:
        verbose_name = 'Verlag'
        verbose_name_plural = 'Verlage'
        ordering = ['code']

    def __str__(self):
        return "{} - {}".format(self.code, self.name)


class ERManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True, publisher__is_active=True)


class EResource(models.Model):
    class ResourceType(models.TextChoices):
        PDF = 'PDF', 'PDF Datei'
        EBOOK = 'EBOOK', 'eBook'
        LINK = 'LINK', 'Link'
        PROXY = 'PROXY', 'WebDB oder Link'
        DB = 'DB', 'Datenbank Citrix'
        UNKNOWN = 'UNKNOWN', 'Unbekannt'

    code = models.CharField('Code', max_length=50, help_text='Eindeutiger Code, Primärschlüssel', unique=True,
                            primary_key=True)
    name = models.CharField('Name', max_length=200, help_text='Bezeichnung der Ressource')
    description = models.CharField('Beschreibung', max_length=255, null=True, blank=True,
                                   help_text='Detailierte Beschreibung wenn vorhanden')
    dbisid = models.CharField('DBIS ID', max_length=20, help_text='DBIS ID für die Datenbank', null=True, blank=True)
    publisher = models.ForeignKey(Publisher, verbose_name='Verlag', on_delete=models.CASCADE, help_text='Verlag')
    restype = models.CharField('Typ', max_length=10, choices=ResourceType.choices,
                               help_text='Welcher Art ist die Ressource')
    licences = models.IntegerField('Lizenzen', help_text='Anzahl Lizenzen absolut', null=True, blank=True,
                                   validators=[MinValueValidator(0), MaxValueValidator(25)])
    tenantuser = models.ManyToManyField(TenantUserGroup, verbose_name='Nutzergruppen',
                                        help_text='Nutzergruppen mit Zugriff')

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)
    is_active = models.BooleanField('Aktiv', default=True, help_text='Ist die Ressource aktiviert')

    objects = models.Manager()
    dbod_objects = ERManager()

    class Meta:
        verbose_name = 'eRessource'
        verbose_name_plural = 'eRessourcen'
        ordering = ['code']

    def __str__(self):
        return "{}: {}".format(self.publisher, self.name)


class EResourceLegacyData(models.Model):
    eresource = models.OneToOneField(EResource, on_delete=models.CASCADE, primary_key=True)
    version = models.CharField('Version', max_length=50, help_text='Version dieser Ressource', null=True, blank=True)
    info = models.CharField('Info', max_length=50, null=True, blank=True,
                            help_text='Was stand im Feld INFO in der IBF Datei')

    def __str__(self):
        return self.eresource.code

    class Meta:
        verbose_name = 'Historisches Datum'
        verbose_name_plural = 'Historische Daten'


class EResourceDBData(models.Model):
    eresource = models.OneToOneField(EResource, on_delete=models.CASCADE, primary_key=True)
    citrixgroup = models.ForeignKey(CitrixWorkerGroup, verbose_name='Worker Group', on_delete=models.SET_NULL,
                                    null=True, blank=True, help_text='Citrix Worker Group')

    def __str__(self):
        return self.eresource.code

    class Meta:
        verbose_name = 'Databank Data'
        verbose_name_plural = 'Datenbank Daten'


class EResourcePDFData(models.Model):
    eresource = models.OneToOneField(EResource, on_delete=models.CASCADE, primary_key=True)
    fileuri = models.CharField('Pfad', max_length=500, null=True, blank=True,
                               help_text='Pfad der PDF-Datei im Dateisystem')

    class Meta:
        verbose_name = 'Details für PDF'
        verbose_name_plural = 'Details für PDF'


class ResourceTenantData(models.Model):
    eresource = models.ForeignKey(EResource, on_delete=models.CASCADE)
    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE)

    limit = models.IntegerField('Limit', null=True, blank=True, help_text='Lizenzlimit für den Mandant')

    link = models.CharField('Link', null=True, blank=True, max_length=500, help_text='Link zur Ressource')
    proxyurl = models.CharField('Proxy', null=True, blank=True, max_length=500,
                                help_text='Proxy URL, link wird hinten angehangen')
    shiburl = models.CharField('Shibboleth', null=True, blank=True, max_length=500, help_text='Shibboleth URL')

    doc_print = models.BooleanField('Drucken', default=False, help_text='Drucken des Dokumentes')
    doc_download = models.BooleanField('Herunterladen', default=False, help_text='Herunterladen des Dokumentes')
    doc_copy = models.BooleanField('Kopieren', default=True, help_text='Kopieren von Text des Dokumentes')
    doc_watermark = models.BooleanField('Wasserzeichen', default=False,
                                        help_text='Wasserzeichen quer über das Dokument')

    class Meta:
        verbose_name = "Mandantendaten"
        verbose_name_plural = "Mandantendaten"
        unique_together = (("eresource", "tenant"),)
