import csv
import logging

from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth import logout
from django.core.paginator import Paginator
from django.db.models import Q
from django.http import Http404, HttpResponse, JsonResponse, HttpResponseRedirect
from django.shortcuts import render, redirect
from django.urls import reverse
from django.utils.http import urlencode
from django.views.decorators.csrf import csrf_exempt
from prometheus_client import Summary

from dbodruntime.models import CitrixSession
from dbodruntime.utils import read_citrix_sessions, explodeAllIPs
from statistic.models import SearchRecord
from statistic.utils import statSearch
from .DBoDMiddleware import TENANTUSERGROUP
from .models import Publisher, EResource, dsettings, TenantUserGroup
from .utils import GUESTS, ANONYMOUS

UPDATE_TIME = Summary('update_processing_seconds', 'Time spent updating sessions')

ANONYM = GUESTS
ANONYMOUS = ANONYMOUS
TUG = TENANTUSERGROUP

logger = logging.getLogger(__name__)


def home(request):
    logger.setLevel(logging.DEBUG)

    query = request.GET.get('q')
    tug = request.session.get(TUG)
    anonymous = tug.code == ANONYMOUS

    if query:
        ressources = EResource.dbod_objects.filter(Q(name__icontains=query) | Q(publisher__name__icontains=query) | Q(
            eresourcelegacydata__info__icontains=query))
    else:
        ressources = EResource.dbod_objects.all()

    if not anonymous:
        ressources = ressources.filter(tenantuser__code__exact=tug)

    logger.debug(ressources.query)

    itemsperpage = dsettings.ITEMSPERPAGE
    paginator = Paginator(ressources, itemsperpage)

    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)

    context = {'view': reverse('home'), 'ressources': page_obj, 'query': query, 'anonymous': anonymous}

    if dsettings.DEVMODE:
        tugs = TenantUserGroup.objects.filter(is_active=True, tenant__is_active=True)
        context.update(devmode=True)
        context.update(tugs=tugs)
        context.update(ipaddr=request.META.get('REMOTE_ADDR'))

    if tug and query:
        statSearch(tug=tug, term=query, results=paginator.count, type=SearchRecord.ALL)

    return render(request, 'dbod/home.html', context)


def publist(request):
    logger.setLevel(logging.DEBUG)

    query = request.GET.get('q')
    page_number = request.GET.get('page')

    tug = request.session.get(TUG)
    tenant = tug.tenant

    anonymous = tug.code == ANONYMOUS

    itemsperpage = dsettings.ITEMSPERPAGE

    if not anonymous:
        ressources = EResource.dbod_objects.filter(tenantuser__code__exact=tug).only('publisher__code')
    else:
        ressources = EResource.dbod_objects.all().only('publisher__code')

    if query:
        publishers = Publisher.dbod_objects.filter(Q(name__icontains=query) | Q(code__icontains=query),
                                                   code__in=ressources.values('publisher__code'))
    else:
        publishers = Publisher.dbod_objects.filter(code__in=ressources.values('publisher__code'))

    logger.debug(publishers.query)

    paginator = Paginator(publishers, itemsperpage)
    page_obj = paginator.get_page(page_number)

    context = {'view': reverse('publist'), 'publishers': page_obj, 'query': query, 'anonymous': anonymous, }

    return render(request, 'dbod/publist.html', context)


def publisher(request, publisher_code):
    logger.setLevel(logging.DEBUG)

    query = request.GET.get('q')
    page_number = request.GET.get('page')

    tug = request.session.get(TUG)
    anonymous = tug.code == ANONYMOUS

    try:
        p = Publisher.objects.get(code=publisher_code)
        if query:
            ressources = EResource.dbod_objects.filter(Q(publisher=p) & (
                    Q(name__icontains=query) | Q(publisher__name__icontains=query) | Q(
                eresourcelegacydata__info__icontains=query)))
        else:
            ressources = EResource.dbod_objects.filter(publisher=p)

        if not anonymous:
            ressources = ressources.filter(tenantuser__code__exact=tug)

        logger.debug(ressources.query)

        itemsperpage = dsettings.ITEMSPERPAGE
        paginator = Paginator(ressources, itemsperpage)

        page_obj = paginator.get_page(page_number)

        context = {'publisher': p, 'view': reverse('publisher', args=(p.code,)), 'ressources': page_obj, 'query': query,
                   'anonymous': anonymous}

        tenant = request.session.get(TUG)
        if tenant and query:
            statSearch(tug=tenant, term=query, results=paginator.count, type=SearchRecord.PUBLISHER)

        return render(request, 'dbod/publisher.html', context)
    except Publisher.DoesNotExist:
        raise Http404("Publisher does not exist")


@csrf_exempt
@UPDATE_TIME.time()
def updatesessions(request):
    if request.method == 'POST':
        if request.body:
            read_citrix_sessions(request.body)
            return JsonResponse({'status': 'OK', 'message': 'Sessions updated'}, status=200)
        else:
            return JsonResponse({'status': 'ERROR', 'message': 'Der Body war Null'}, status=500)
    else:
        return JsonResponse({'status': 'ERROR', 'message': 'Kein POST'}, status=405)


@staff_member_required
def citrixsessions(request):
    sessions = CitrixSession.objects.all()
    return render(request, 'dbod/admin/citrixsessions.html', {'sessions': sessions})


def aduserlist(request):
    # Eine Liste aller Ressourcen gefiltert nach Aktiv und DB
    res_list = EResource.objects.filter(is_active=True, restype=EResource.ResourceType.DB)

    # Konstruieren der Response
    resp = HttpResponse(content_type='text/csv')
    resp['Content-Disposition'] = 'attachment; filename="ad-userlist.csv"'

    # Einen CSV-Writer
    writer = csv.writer(resp)
    # Kopfzeile
    writer.writerow(['name', 'num_lic'])
    # Loop über alle Objekte in der Abfrage oben und jeweils eine Zeile aus Code und Anzahl
    for er in res_list:
        writer.writerow([er.code, er.licences])

    # Fertig
    return resp


def cacheips(request):
    c = explodeAllIPs()
    return render(request, 'dbod/admin/refreships.html', context={'ipcount': c})


def shiblogin(request):
    ret = request.GET.get('return')
    if ret:
        return HttpResponseRedirect(dsettings.SHIBLOGINURL + '?' + urlencode({'return': ret}))
    else:
        return HttpResponseRedirect(dsettings.SHIBLOGINURL + '?' + urlencode({'return': reverse('home')}))


def shiblogout(request):
    logout(request)
    ret = request.GET.get('return')
    if ret:
        return HttpResponseRedirect(dsettings.SHIBLOGOUTURL + '?' + urlencode({'return': ret}))
    else:
        return HttpResponseRedirect(dsettings.SHIBLOGOUTURL + '?' + urlencode({'return': reverse('home')}))


def selfserviceredirects(request):
    url = request.path
    url = url.replace('/selfservice/', '/mandant/')
    return redirect(url)
