import ipaddress

from django.contrib import admin
from django.db.models import Count
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from dbodruntime.models import CitrixSession, CitrixWorker, CitrixWorkerGroup, WorkerJob, ExplodedIPs, DocWorkerJob
from dbodruntime.utils import delete_dead_docworker


class CitrixSessionAdmin(admin.ModelAdmin):
    list_display = ['code', 'realuser', 'machine', 'connectionip', 'starttime']


class WorkerAdmin(admin.ModelAdmin):
    list_display = ['code', 'name', 'group_link', 'is_active', 'is_running']
    list_filter = ['is_active', 'is_running', ]

    def group_link(self, obj):
        if obj.group:
            return format_html('<a href="{}">{}</a>'.format(
                reverse("admin:dbodruntime_citrixworkergroup_change", kwargs={'object_id': obj.group.code}), obj.group))
        else:
            return 'Ohne Gruppe'

    group_link.short_description = _("Group")

    def stoprunning(self, request, queryset):
        queryset.update(is_running=False)

    actions = ['stoprunning']


class WorkerGroupAdmin(admin.ModelAdmin):
    list_display = ['code', 'name', 'worker_count', 'is_active']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = queryset.annotate(count_worker=Count('citrixworker'))
        return queryset

    def worker_count(self, obj):
        baseurl = reverse("admin:dbodruntime_citrixworker_changelist") + '?group__code__exact=' + obj.code
        return format_html('<a href="{}">{}</a>', baseurl, obj.count_worker)

    worker_count.short_description = _("Worker")


class WorkerJobAdmin(admin.ModelAdmin):
    list_display = ['id', 'worker', 'eresource', 'connectionip', 'created', 'lastupdate']


class DocWorkerJobAdmin(admin.ModelAdmin):
    list_display = ['id', 'eresource', 'connectionip', 'created', 'lastupdate']

    def delete_dead(self, request, queryset):
        delete_dead_docworker()

    actions = ['delete_dead']


class ExplodedIPAddressAdmin(admin.ModelAdmin):
    list_display = ['tenantusergroup', 'tenant', 'ipaddress_min', 'ipaddress_max', 'cdir_min', 'cdir_max']
    list_filter = ['tenantusergroup']

    def tenant(self, obj):
        return obj.tenantusergroup.tenant

    def cdir_min(self, obj):
        ipnum = obj.ipaddress_min
        cdir = ipaddress.ip_address(ipnum)
        return cdir.exploded

    def cdir_max(self, obj):
        ipnum = obj.ipaddress_max
        cdir = ipaddress.ip_address(ipnum)
        return cdir.exploded


admin.site.register(CitrixSession, CitrixSessionAdmin)
admin.site.register(CitrixWorker, WorkerAdmin)
admin.site.register(CitrixWorkerGroup, WorkerGroupAdmin)
admin.site.register(WorkerJob, WorkerJobAdmin)
admin.site.register(DocWorkerJob, DocWorkerJobAdmin)
admin.site.register(ExplodedIPs, ExplodedIPAddressAdmin)
