import re

import dbsettings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class DBoDRuntimeSettings(dbsettings.Group):
    CITRIXURL = dbsettings.StringValue(
        help_text='Pfad für Citrix Ranch',
        default='https://sdvdbodranch02.slub-dresden.de/Citrix/StoreWeb/SDK.html'
    )

    PDFREADER = dbsettings.StringValue(
        help_text='Pfad zum PDF Reader',
        default='http://localhost:8080/'
    )

    PDFLICENCE = dbsettings.StringValue(
        help_text='Lizenz des PDF Express Readers für die alktuelle Domain',
        default='qXj5X8hwG4YBxcgmLZTc'
    )

    PDFBASEPATH = dbsettings.StringValue(
        help_text='Basispfad für PDF Dateien',
        default='pdflib/'
    )

    DEADTIME = dbsettings.IntegerValue(
        help_text='Zeit in Sekunden, nach denen ein DocWorker gelöscht wird',
        default=60
    )


dsettings = DBoDRuntimeSettings('Laufzeit Einstellungen')


class CitrixSession(models.Model):
    code = models.CharField(max_length=50, primary_key=True, null=False, blank=False, help_text='Sessionskey',
                            editable=False)
    user = models.CharField(max_length=50, null=False, blank=False, editable=False, help_text='User der Session')

    connectionip = models.GenericIPAddressField(blank=True, null=True, help_text='Verbindungs IP', editable=False)
    machine = models.CharField(max_length=200, blank=True, null=True, help_text='Maschinenname', editable=False)
    starttime = models.DateTimeField(help_text='Startzeit', editable=False, null=True)
    is_updated = models.BooleanField(help_text='Wurde es im aktuellen Lauf erneuert?', default=True)
    is_verified = models.BooleanField(help_text='Wurde per Logon bestätigt', default=False)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = 'Citrix Session'
        verbose_name_plural = 'Citrix Sessions'

    @property
    def realuser(self):
        regex = r"^(?:.*\\)?(?P<username>.*)$"
        u = re.search(regex, self.user)
        if u.group('username'):
            return u.group('username')
        else:
            return self.user

    def __str__(self):
        return "{}/{}".format(self.machine, self.user)


class CitrixWorkerGroup(models.Model):
    code = models.CharField(max_length=50, primary_key=True, null=False, blank=False, help_text=_('Code or something'))
    name = models.CharField(max_length=50, help_text=_('Readable name'), default=code)
    is_active = models.BooleanField(default=True, help_text=_('This is a active group'))

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "{} ({})".format(self.name, self.code)

    class Meta:
        verbose_name = 'Citrix Worker Group'
        verbose_name_plural = 'Citrix Worker Groups'


class CitrixWorker(models.Model):
    code = models.CharField(max_length=50, primary_key=True, null=False, blank=False,
                            help_text=_('Code or something'))
    name = models.CharField(max_length=50, help_text=_('Readable name for the group'))
    is_running = models.BooleanField(help_text=_('This worker is running'), default=False)
    is_active = models.BooleanField(default=True, help_text=_('This is a active worker'))
    group = models.ForeignKey(CitrixWorkerGroup, on_delete=models.SET_NULL, null=True, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "{} ({})".format(self.name, self.code)

    class Meta:
        verbose_name = 'Citrix Worker'
        verbose_name_plural = 'Citrix Workers'


class WorkerJob(models.Model):
    worker = models.ForeignKey(CitrixWorker, on_delete=models.CASCADE)
    eresource = models.ForeignKey('DBoD.EResource', on_delete=models.CASCADE)
    tenant = models.ForeignKey('DBoD.Tenant', null=True, blank=True, on_delete=models.CASCADE)
    connectionip = models.GenericIPAddressField(blank=True, null=True, help_text='Verbindungs IP', editable=False)
    session = models.ForeignKey(CitrixSession, on_delete=models.SET_NULL, null=True, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = (('worker', 'eresource'),)
        verbose_name = 'Worker Job'
        verbose_name_plural = 'Worker Jobs'


class DocWorkerJob(models.Model):
    eresource = models.ForeignKey('DBoD.EResource', on_delete=models.CASCADE)
    connectionip = models.GenericIPAddressField(blank=True, null=True, help_text='Verbindungs IP', editable=False)
    tenantusergroup = models.ForeignKey('DBoD.TenantUserGroup', on_delete=models.CASCADE)
    tenant = models.ForeignKey('DBoD.Tenant', null=True, blank=True, on_delete=models.CASCADE)

    created = models.DateTimeField(auto_now_add=True)
    lastupdate = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = 'Dokumenten Job'
        verbose_name_plural = 'Dokumenten Jobs'


class ExplodedIPs(models.Model):
    tenantusergroup = models.ForeignKey('DBoD.TenantUserGroup', on_delete=models.CASCADE)
    ipaddress_min = models.BigIntegerField(help_text='Minimale IP')
    ipaddress_max = models.BigIntegerField(help_text='Maximale IP')
    prio = models.IntegerField(help_text='Priorität', default=10)

    class Meta:
        verbose_name = 'IP / Nutzergruppe'
        verbose_name_plural = 'IPs / Nutzergruppen'
