import logging

from prometheus_client import Gauge

from DBoD.models import EResource, TenantUserGroup, Tenant
from dbodruntime.models import CitrixWorker, WorkerJob, CitrixSession, DocWorkerJob

g = Gauge('citrix_session', 'Citrix Session used')
d = Gauge('doc_session', 'Document Session used')
logger = logging.getLogger(__name__)


class SessionManager(object):
    def __init__(self):
        g.set(0)
        d.set(0)

    def findJ4User(self, username, connectionip):
        try:
            job = WorkerJob.objects.get(worker__code__exact=username, connectionip__exact=connectionip)
            logger.info('Found job for {} with IP {} - Job-ID: {}'.format(username, connectionip, job.id))
            return job
        except WorkerJob.DoesNotExist:
            logger.warning(
                'findJ4User: Can\'t find job with connection IP {}, try to find another.'.format(connectionip))
            job = WorkerJob.objects.filter(worker__code__exact=username).first()
            if job:
                return job
            else:
                logger.error('findJ4User: Can\'t find job for {}'.format(username))
                return None

    def add_job(self, worker: CitrixWorker, eres: EResource, tenant: Tenant, reqip: str):
        job = WorkerJob(worker=worker, eresource=eres, tenant=tenant)
        worker.is_running = True
        worker.save()
        job.connectionip = reqip
        job.save()
        g.inc()
        return job

    def add_docjob(self, eres: EResource, reqip: str, tug: TenantUserGroup, tenant: Tenant):
        job = DocWorkerJob(eresource=eres, connectionip=reqip, tenantusergroup=tug, tenant=tenant)
        job.save()
        g.inc()
        return job

    def del_docjob(self, jobid: str):
        pass

    def verify_session(self, sessionkey: str, connectionip: str, username: str):
        try:
            session = CitrixSession.objects.get(code=sessionkey)
            session.is_verified = True
            session.save()
            logger.info('Session {} found, verified.')
            return session
        except CitrixSession.DoesNotExist:
            logger.info("Session {} not found, create one.".format(sessionkey))
            session = CitrixSession(code=sessionkey, connectionip=connectionip, user=username)
            session.is_verified = True
            session.save()
            return session

    def deregister_session(self, sessionkey: str, connectionip: str, username: str):
        try:
            eres = None
            session: CitrixSession = CitrixSession.objects.get(code=sessionkey)
            job: WorkerJob = session.workerjob_set.first()
            if job:
                eres = job.eresource
                worker: CitrixWorker = job.worker
                worker.is_running = False
                worker.save()
                job.delete()
            else:
                logger.error('Session {} hat keinen Worker.'.format(sessionkey))

            session.is_verified = False
            session.save()

            g.dec()

            return eres
        except CitrixSession.DoesNotExist:
            logger.error('Session {} is not in database (from logoff).'.format(sessionkey))
            return None
        except WorkerJob.DoesNotExist:
            logger.error('Session {} hat keinen Worker.'.format(sessionkey))
            return None
        pass


session_manager = SessionManager()
