import ipaddress
import json
import logging
import re
from datetime import datetime, timedelta

from django.utils.timezone import now

from DBoD.models import TenantUserGroup, TenantUserGroupIPAuth
from dbodruntime.models import CitrixSession, WorkerJob, CitrixWorker, ExplodedIPs, DocWorkerJob, DBoDRuntimeSettings
from statistic.utils import statPageView

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def deletesessions(sessions):
    for session in sessions:
        try:
            job: WorkerJob = WorkerJob.objects.get(session=session)
            worker: CitrixWorker = job.worker
            logger.info('Lösche Job {} und gebe Worker {} frei.'.format(job.id, worker.code))
            worker.is_running = False
            worker.save()
            job.delete()
        except WorkerJob.DoesNotExist:
            logger.warning('Session {} hatte keinen Worker (mehr).'.format(session.code))

        logger.info('Lösche Session {}'.format(session.code))
        session.delete()


def read_citrix_sessions(body: str):
    sessions = json.loads(body)
    CitrixSession.objects.all().update(is_updated=False)
    for row in sessions:
        code = row['SessionKey']

        user = row['UserName']
        regex = r"^(?:.*\\)?(?P<username>.*)$"
        u = re.search(regex, user)
        if u and u.group('username'):
            user = u.group('username')

        machine = row['MachineName']
        connectionip = row['ConnectedViaIP']

        starttime = row['StartTime']
        # dateregex = r"\\/Date\((?P<datum>\d+)\)\\/"
        dateregex = r"^.*(?P<datum>\d{13}).*$"
        st = re.search(dateregex, starttime)
        if st and st.group('datum'):
            starttime = datetime.fromtimestamp(int(st.group('datum')) / 1000).strftime('%Y-%m-%d %H:%M:%S.%f')

        session, created = CitrixSession.objects.get_or_create(code=code)
        session.user = user
        session.machine = machine
        session.connectionip = connectionip
        session.starttime = starttime
        session.is_updated = True
        session.save()

        if created:
            logger.info('Session {} gespeichert.'.format(code))
        else:
            logger.info('Session {} aktualisiert.'.format(code))

    old_sessions = CitrixSession.objects.filter(is_updated=False)
    deletesessions(old_sessions)


def explodeTUGIPs(tug: TenantUserGroup):
    logger.info('Caching {} for {}'.format(tug.name, tug.tenant.name))
    ips = TenantUserGroupIPAuth.objects.filter(tenantusergroup=tug, is_active=True)
    for ip in ips:
        ipauth: TenantUserGroupIPAuth = ip
        try:
            if ipauth.network:
                net = ipaddress.ip_network(ipauth.ipaddr)
                min = list(net.hosts())[0]
                max = list(net.hosts())[-1]
                prio = ipauth.prio
                eip = ExplodedIPs(tenantusergroup=tug, ipaddress_min=int(min), ipaddress_max=int(max), prio=prio)
                eip.save()
            else:
                host = ipaddress.ip_address(ipauth.ipaddr)
                hostaddr = host
                prio = ipauth.prio
                eip = ExplodedIPs(tenantusergroup=tug, ipaddress_min=int(hostaddr), ipaddress_max=int(hostaddr),
                                  prio=prio)
                eip.save()
        except ValueError:
            logger.error('TenantUserGroup {} has an invalid address!'.format(tug.name))
    return ips.count()


def explodeAllIPs():
    ExplodedIPs.objects.all().delete()

    tugs = TenantUserGroup.objects.filter(is_active=True)
    ips = 0

    for tug in tugs:
        ips += explodeTUGIPs(tug=tug)
    return ips


def refresh_worker(jobid):
    try:
        job = DocWorkerJob.objects.get(id=jobid)
        job.lastupdate = now()
        job.save()
        return True
    except DocWorkerJob.DoesNotExist:
        logger.error('DocWorker mit ID {} nicht gefunden.'.format(jobid))
        return False


def delpdfworker(jobid):
    try:
        job = DocWorkerJob.objects.get(id=jobid)
        job.delete()
        logger.info('PDF Worker {} gelöscht.'.format(jobid))
        return True
    except DocWorkerJob.DoesNotExist:
        logger.error('DocWorker mit ID {} nicht gefunden.'.format(jobid))
        return False


def recStat(page, jobid):
    try:
        job = DocWorkerJob.objects.get(id=jobid)
        tug = job.tenantusergroup
        eres = job.eresource
        statPageView(tug=tug, eres=eres, page=page)
        return True
    except DocWorkerJob.DoesNotExist:
        logger.error('DocWorker mit ID {} nicht gefunden.'.format(jobid))
        return False


def delete_dead_docworker():
    settings = DBoDRuntimeSettings()
    deadtime = settings.DEADTIME
    olderthan = now() - timedelta(seconds=deadtime)
    deleted, count = DocWorkerJob.objects.filter(lastupdate__lte=olderthan).delete()
    return deleted


def delete_dead_citrixworker():
    settings = DBoDRuntimeSettings()
    deadtime = settings.DEADTIME
    olderthan = now() - timedelta(seconds=deadtime)
    deleted, count = WorkerJob.objects.filter(lastupdate__lte=olderthan).delete()
    return deleted
