import json
import logging
import urllib.parse

from django.contrib import messages
from django.http import JsonResponse, FileResponse
from django.shortcuts import render, redirect
from django.urls import reverse
from django.views.decorators.csrf import csrf_exempt

from DBoD.DBoDMiddleware import TENANTUSERGROUP
from DBoD.models import EResource, ResourceTenantData
from DBoD.views import TUG
from dbodruntime.models import CitrixWorker, DBoDRuntimeSettings, DocWorkerJob, WorkerJob
from dbodruntime.sessionhandler import session_manager
from dbodruntime.utils import refresh_worker, recStat, delpdfworker
from statistic.utils import statOpen, statDB

logger = logging.getLogger(__name__)

TUG = TENANTUSERGROUP


def startworker(request, resource):
    logger.setLevel(logging.DEBUG)
    logger.debug('>>> START WORKER')
    context = {}
    reqip = request.META['REMOTE_ADDR']
    payload = None
    tug = request.session.get(TUG)
    settings = DBoDRuntimeSettings()

    try:
        res: EResource = EResource.dbod_objects.get(code=resource)
        context.update(code=res.code, name=res.name)
        eres = None
        worker = None

        # Schauen ob das überhaupt eine DB ist:
        if res.restype == EResource.ResourceType.DB:
            eres = res
            rtd = ResourceTenantData.objects.filter(eresource=res, tenant=tug.tenant, limit__gt=0)
            if rtd.count() == 1:
                limit = rtd.first().limit
                cur = WorkerJob.objects.filter(eresource=res, tenant=tug.tenant).count()
                if cur >= limit:
                    context.update(
                        error='Die Anzahl gleichzeitiger Nutzungen der Ressource ist begrenzt und keine freie Nutzungslizenz ist mehr vorhanden.'
                    )
                    return render(request, 'dbodruntime/error_startworker.html', context)

            # Wenn es eine Gruppe erfordert:
            if res.eresourcedbdata.citrixgroup:
                group = res.eresourcedbdata.citrixgroup
                worker = CitrixWorker.objects.filter(group=group, is_running=False).first()
            # Wenn nicht:
            else:
                worker = CitrixWorker.objects.filter(group=None, is_running=False).first()

            # Schauen ob es einen Worker gab:
            if not worker:
                context.update(error='Keine freien Worker für die DB!')
                return render(request, 'dbodruntime/error_startworker.html', context)
            else:
                context.update(workername=worker.code)

            try:
                job = session_manager.add_job(worker=worker, eres=res, tenant=tug.tenant, reqip=reqip)
                context.update(job=job)
            except Exception as e:
                messages.add_message(request, messages.ERROR, 'Bei der Joberzeugung ging was schief!')
        elif res.restype == EResource.ResourceType.PDF:
            eres = res

            rtd = ResourceTenantData.objects.filter(eresource=res, tenant=tug.tenant, limit__gt=0)
            if rtd.count() == 1:
                limit = rtd.first().limit
                cur = DocWorkerJob.objects.filter(eresource=eres, tenant=tug.tenant).count()
                if cur >= limit:
                    context.update(
                        error='Die Anzahl gleichzeitiger Nutzungen der Ressource ist begrenzt und keine freie Nutzungslizenz ist mehr vorhanden. Versuchen sie es später noch einmal.'
                    )
                    return render(request, 'dbodruntime/error_startworker.html', context)

            job = session_manager.add_docjob(eres=eres, reqip=reqip, tug=tug, tenant=tug.tenant)
            params = {'jobid': job.id, }
            return redirect(reverse('pdfviewer') + '?' + urllib.parse.urlencode(params))
        else:
            messages.add_message(request, messages.ERROR, 'Die Resource ist kein Typ der einen Starter hat!')
            context.update(worker=worker, error='Die Resource ist keine Datenbank!')
            return render(request, 'dbodruntime/error_startworker.html', context)
    except CitrixWorker.DoesNotExist as e:
        context.update(
            error='Fehler! DBoD hat im Moment keine freien Sessions um weitere Datenbanken zu starten. Versuchen sie es später noch einmal.')
        return render(request, 'dbodruntime/error_startworker.html', context)
    except EResource.DoesNotExist:
        context.update(error='Fehler! Die Resource gibt es nicht im System.')
        return render(request, 'dbodruntime/error_startworker.html', context)
    except ResourceTenantData.DoesNotExist as e:
        context.update(error='Für die angeforderte Resource gibt es keine Informationen zu Rechten.')
        return render(request, 'dbodruntime/error_startworker.html', context)

    url = settings.CITRIXURL

    params = {'nutzer': 'SLUB\\' + job.worker.code, }

    if tug:
        statOpen(tug=tug, eres=eres)

    return redirect(url + '?' + urllib.parse.urlencode(params))


@csrf_exempt
def getworkerjob(request):
    logger.setLevel(logging.DEBUG)
    logger.debug('>>> START LOGON')
    if request.method == 'POST':
        logger.debug(request.body)
        data = json.loads(request.body)
        logger.debug(data)
        if data['username'] and data['sessionkey'] and data['connectionip']:
            payload = {}
            payload.update(connectionip=data['connectionip'])
            job = session_manager.findJ4User(username=data['username'], connectionip=data['connectionip'])
            session = session_manager.verify_session(sessionkey=data['sessionkey'], connectionip=data['connectionip'],
                                                     username=data['username'])
            if session:
                payload.update(sessionkey=session.code)
                payload.update(citrixip=session.connectionip)

            if job:
                payload.update(database=job.eresource.code)
                if job and session:
                    job.session = session
                    job.save()
                return JsonResponse(payload, status=200)
            else:
                logger.error(data)
                return JsonResponse({'database': 'ERROR', 'message': 'keine DB für User {}'.format(data['username'])},
                                    status=404)
        else:
            logger.error(data)
            return JsonResponse({'database': 'ERROR', 'message': 'nicht ausreichende Daten'}, status=500)
    else:
        return JsonResponse({'database': 'ERROR', 'message': 'kein POST'}, status=405)


@csrf_exempt
def delworker(request):
    logger.setLevel(logging.DEBUG)
    logger.debug('>>> START LOGOFF')

    tug = request.session.get(TUG)

    if request.method == 'POST':
        logger.debug(request.body)
        data = json.loads(request.body)
        logger.debug(data)

        if not data['runtime']:
            data['runtime'] = 0

        if all(key in data for key in ('username', 'sessionkey', 'connectionip', 'runtime')):
            res = session_manager.deregister_session(sessionkey=data['sessionkey'], connectionip=data['connectionip'],
                                                     username=data['username'])
            if res:
                statDB(tug=tug, eres=res, runtime=data['runtime'])
                return JsonResponse({'status': 'OK'}, status=200)
            else:
                return JsonResponse({'status': 'ERROR', 'message': 'Ein Fehler trat beim Deaktivieren auf, check log!'},
                                    status=500)
        else:
            return JsonResponse({'status': 'ERROR', 'message': 'nicht ausreichende parameter'}, status=500)
    else:
        return JsonResponse({'status': 'ERROR', 'message': 'kein POST'}, status=405)


def pdfinfo(request, jobid):
    settings = DBoDRuntimeSettings()
    LICENCE = settings.PDFLICENCE
    job = DocWorkerJob.objects.get(id=jobid)
    tug = job.tenantusergroup
    res = job.eresource

    if res.restype == EResource.ResourceType.PDF:
        rtd = ResourceTenantData.objects.get(eresource=res, tenant=tug.tenant)
        file = res.eresourcepdfdata.fileuri

        print = True if rtd.doc_print else False
        copy = True if rtd.doc_copy else False
        download = True if rtd.doc_download else False
        watermark = True if rtd.doc_watermark else False

        info = {'license': LICENCE, 'pdffile': reverse('pdffile', args=[file]), 'print': print, 'copypaste': copy,
                'download': download, 'watermark': watermark, }
    else:
        info = {'license': LICENCE, 'pdffile': reverse('pdffile', args=['notvalid.pdf']), 'print': False,
                'copypaste': False, 'download': False, 'watermark': False}
    return JsonResponse(info, status=200)


def pdffile(request, filename):
    settings = DBoDRuntimeSettings()
    PDFBASEPATH = settings.PDFBASEPATH
    return FileResponse(open(PDFBASEPATH + filename, 'rb'), as_attachment=False, content_type='application/pdf')


@csrf_exempt
def heartbeat(request):
    logger.setLevel(logging.DEBUG)
    if request.method == 'POST':
        data = json.loads(request.body)
        page = data['pagenumber']
        jobid = data['jobid']
        logger.debug('Herzschlag: Job {}, Seite: {}'.format(jobid, page))
        if jobid:
            if page == -1:
                refresh_worker(jobid)
            else:
                refresh_worker(jobid)
                recStat(page, jobid)
        return JsonResponse({'status': 'ok'}, status=200)
    else:
        return JsonResponse({'status': 'ERROR', 'message': 'Methode nicht erlaubt!'}, status=405)


def pdfviewer(request):
    return render(request, template_name='index.html')


def shibb(request):
    pass
    return None


@csrf_exempt
def deldocjob(request):
    logger.setLevel(logging.DEBUG)
    if request.method == 'POST':
        data = json.loads(request.body)
        jobid = data['jobid']
        logger.debug('Logoff PDF: Job {}'.format(jobid))
        if jobid:
            delpdfworker(jobid)
        return JsonResponse({'status': 'ok'}, status=200)
    else:
        return JsonResponse({'status': 'ERROR', 'message': 'Methode nicht erlaubt!'}, status=405)
