# Register your models here.
import pathlib
import shutil

import unicodedata
from django.contrib import admin
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.urls import path, reverse
from django.utils.html import format_html

from DBoD.models import Publisher, TenantUserGroup, EResource, EResourcePDFData, ResourceTenantData
from dbodruntime.models import DBoDRuntimeSettings
from tenantservice.forms import AcceptUploadForm
from tenantservice.models import Tug2ResApplication, UploadFile, ContactData, ReportData, GuestContactData


class Tug2ResApplicationAdmin(admin.ModelAdmin):
    list_display = ['tenant', 'eresource', 'is_new', 'is_approved', 'is_done']
    list_filter = ['is_new', 'is_approved', 'is_done']


class UploadFileAdmin(admin.ModelAdmin):
    list_display = ['iname', 'accept_res', 'file', 'ftp_upload', 'tenant', 'is_new', 'is_approved', 'is_done']
    list_filter = ['tenant', 'is_new', 'is_approved', 'is_done']

    def accept_res(self, obj):
        if not obj.is_done:
            return format_html('<a href="{}/accept/">{}</a>'.format(obj.id, 'Akzeptieren'))
        else:
            return 'Bereits akzeptiert/bearbeitet'

    accept_res.short_description = 'Akzeptieren'

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [path('<int:object_id>/accept/', self.admin_site.admin_view(self.accept), name='accept'), ]
        return my_urls + urls

    def accept(self, request, object_id):
        upload = UploadFile.objects.get(id=object_id)
        if request.method == 'POST':
            if upload.ftp_upload:
                form = AcceptUploadForm(tenant=upload.tenant, data=request.POST)
                context = dict(self.admin_site.each_context(request), res_form=form, )
                return TemplateResponse(request, "admin/accept_ftp.html", context)
            else:
                form = AcceptUploadForm(tenant=upload.tenant, data=request.POST)

                if form.is_valid():
                    code = form.cleaned_data.get('code')
                    code = code.strip().lower()

                    name = form.cleaned_data.get('name')
                    description = form.cleaned_data.get('description')
                    dbisid = form.cleaned_data.get('dbis')

                    publisher_code = form.cleaned_data.get('publisher')
                    publisher = Publisher.objects.get(code=publisher_code)

                    type = form.cleaned_data.get('type')
                    restype = EResource.ResourceType(type)

                    tenantusers = form.cleaned_data.get('tenantusers')
                    tugs = TenantUserGroup.objects.filter(code__in=tenantusers)

                    is_active = form.cleaned_data.get('active')

                    if code == 'auto':
                        namecode = name.lower().strip().replace(' ', '').replace('ß', 'ss')
                        namecode = unicodedata.normalize('NFKD', namecode).encode('ascii', 'ignore').decode('ascii')
                        namecode = namecode.ljust(8, u'x')
                        namecode = namecode[:8]
                        code = '{}_{}'.format(publisher_code, namecode)

                    res = EResource(code=code)
                    res.name = name
                    res.description = description
                    res.dbisid = dbisid
                    res.publisher = publisher
                    res.restype = restype
                    res.tenantuser.add(*tugs)
                    res.is_active = is_active

                    res.save()

                    settings = DBoDRuntimeSettings()

                    if res.restype == EResource.ResourceType.PDF:
                        src = pathlib.Path(upload.file)
                        tgt = pathlib.Path(settings.PDFBASEPATH)
                        shutil.copy(src, tgt)
                        pdf = EResourcePDFData(eresource=res)
                        pdf.fileuri = src.name
                        pdf.save()

                    rtd = ResourceTenantData(eresource=res, tenant=upload.tenant)
                    rtd.limit = form.cleaned_data.get('licences')
                    rtd.save()

                    upload.is_done = True
                    upload.save()

                    return redirect(reverse('admin:DBoD_eresource_change', kwargs={'object_id': res.code}))

                else:
                    context = dict(self.admin_site.each_context(request), upload=upload, res_form=form, )
                    return TemplateResponse(request, "admin/accept_direkt.html", context)
        else:
            if upload.ftp_upload:
                form = AcceptUploadForm(tenant=upload.tenant)
                context = dict(self.admin_site.each_context(request), res_form=form, )
                return TemplateResponse(request, "admin/accept_ftp.html", context)
            else:
                form = AcceptUploadForm(tenant=upload.tenant)
                form.fields['code'].initial = 'auto'
                form.fields['name'].initial = upload.iname
                form.fields['description'].initial = upload.idesc
                form.fields['dbis'].initial = upload.idbis
                form.fields['licences'].initial = upload.iuser
                form.fields['tenantusers'].initial = [tug.code for tug in upload.tenantusergroups.all()]

                context = dict(self.admin_site.each_context(request), upload=upload, res_form=form, )
                return TemplateResponse(request, "admin/accept_direkt.html", context)


class ContactDataAdmin(admin.ModelAdmin):
    list_display = ['tenantuser', 'subject']
    list_filter = ['tenant']
    search_fields = ['subject', 'text']


class GuestContactDataAdmin(admin.ModelAdmin):
    list_display = ['subject', 'created']
    search_fields = ['subject', 'text']


class ReportDataAdmin(admin.ModelAdmin):
    list_display = ['tenantuser', 'eresource', 'subject']
    list_filter = ['tenant', 'eresource']
    search_fields = ['subject', 'text']


admin.site.register(Tug2ResApplication, Tug2ResApplicationAdmin)
admin.site.register(UploadFile, UploadFileAdmin)
admin.site.register(ContactData, ContactDataAdmin)
admin.site.register(GuestContactData, GuestContactDataAdmin)
admin.site.register(ReportData, ReportDataAdmin)
