from captcha.fields import CaptchaField
from django import forms
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError

from DBoD.models import TenantUser, Tenant, TenantUserGroup, EResource, Publisher
from tenantservice.models import Tug2ResApplication, ContactData, ReportData, GuestContactData


class TenantUserForm(forms.ModelForm):
    class Meta:
        model = TenantUser
        fields = ['tel', 'is_tenantadmin', 'is_active']


class OwnUserForm(forms.ModelForm):
    class Meta:
        model = TenantUser
        fields = ['tel']


class UserForm(forms.ModelForm):
    password = forms.CharField(
        widget=forms.PasswordInput(),
        help_text='Neues Passwort',
        initial=None,
        label='Passwort',
        required=False
    )
    confirm_password = forms.CharField(
        widget=forms.PasswordInput(),
        help_text='Wiederholung neues Passwort',
        initial=None, label='Passwort Wiederholung',
        required=False
    )

    class Meta:
        model = User
        fields = ['first_name', 'last_name', 'email', 'username']

    def clean(self):
        cleaned_data = super(UserForm, self).clean()
        password = cleaned_data.get("password")
        confirm_password = cleaned_data.get("confirm_password")

        if password != confirm_password:
            self.add_error('confirm_password', "Die Passwörter stimmen nicht überein!")

        return cleaned_data


class TenantForm(forms.ModelForm):
    class Meta:
        model = Tenant
        fields = ['name', 'desc', 'tel', 'email', 'dbis', ]


class TenantUserGroupForm(forms.ModelForm):
    class Meta:
        model = TenantUserGroup
        fields = ['name', 'desc', 'shibb_affiliate', 'is_active']


class AppTug2ResForm(forms.ModelForm):
    class Meta:
        model = Tug2ResApplication
        fields = ['tenantusergroup', 'text']


class DirectUploadForm(forms.Form):
    name = forms.CharField(
        help_text='Kurze Bezeichnung',
        label='Bezeichnung'
    )
    desc = forms.CharField(
        help_text='Beschreibung',
        label='Beschreibung',
        required=False
    )

    publisher_select = forms.ChoiceField(
        help_text='Aus vorhandenen Verlagen auswählen',
        label='Verlag',
        required=False,
    )

    publisher = forms.CharField(
        help_text='Verlag',
        label='Anderer Verlag, der oben nicht ausgewählt werden kann',
        required=False,
    )

    dbis = forms.CharField(
        help_text='DBID ID, wenn vorhanden.',
        label='DBIS ID',
        required=False
    )
    usercount = forms.IntegerField(
        help_text='Anzahl gleichzeitiger Nutzer, wenn gegeben.',
        label='Nutzerlizenzen',
        required=False
    )
    freetext = forms.CharField(
        help_text='Installationshinweise, allgemeine Hinweise,...',
        widget=forms.Textarea,
        label='Freitext',
        required=False
    )
    tenantusers = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple)

    file = forms.FileField(help_text='Datei')

    def __init__(self, tenant, *args, **kwargs):
        super(DirectUploadForm, self).__init__(*args, **kwargs)
        self.fields['tenantusers'] = forms.MultipleChoiceField(
            label='Nutzergruppen mit Zugriff',
            widget=forms.CheckboxSelectMultiple,
            choices=[
                (tug.code, tug.name) for tug in
                TenantUserGroup.objects.filter(
                    tenant=tenant,
                    is_active=True
                )
            ])
        self.fields['publisher_select'].choices = tuple(
            [(u'', 'Keine Auswahl')] + list(Publisher.objects.all().values_list('code', 'name', named=False)))

    def clean(self):
        cleaned_data = super().clean()

        pub = cleaned_data['publisher']
        pub_sel = cleaned_data['publisher_select']

        if pub and pub_sel:
            msg = 'Bitte wählen sie entweder einen Verlag aus ODER tragen sie einen neuen in das andere Feld ein!'
            self.add_error('publisher', msg)
            self.add_error('publisher_select', msg)
            raise ValidationError(msg)


class FtpUploadForm(forms.Form):
    name = forms.CharField(
        help_text='Kurze Bezeichnung',
        label='Bezeichnung'
    )
    desc = forms.CharField(
        help_text='Beschreibung',
        label='Beschreibung',
        required=False
    )

    publisher_select = forms.ChoiceField(
        help_text='Aus vorhandenen Verlagen auswählen',
        label='Verlag',
        required=False,
    )

    publisher = forms.CharField(
        help_text='Verlag',
        label='Anderer Verlag, der oben nicht ausgewählt werden kann',
        required=False,
    )

    dbis = forms.CharField(
        help_text='DBID ID, wenn vorhanden.',
        label='DBIS ID',
        required=False
    )
    usercount = forms.IntegerField(
        help_text='Anzahl gleichzeitiger Nutzer, wenn gegeben.',
        label='Nutzerlizenzen',
        required=False
    )
    freetext = forms.CharField(
        help_text='Installationshinweise, allgemeine Hinweise,...',
        widget=forms.Textarea,
        label='Freitext',
        required=False
    )
    tenantusers = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple)

    filename = forms.CharField(
        help_text='Bitte geben sie den Dateinamen an, damit wir die Datei schnell auf dem Server finden',
        label='Dateiname'
    )

    def __init__(self, tenant, *args, **kwargs):
        super(FtpUploadForm, self).__init__(*args, **kwargs)
        self.fields['tenantusers'] = forms.MultipleChoiceField(
            label='Nutzergruppen mit Zugriff',
            widget=forms.CheckboxSelectMultiple,
            choices=[
                (tug.code, tug.name) for tug in
                TenantUserGroup.objects.filter(
                    tenant=tenant,
                    is_active=True
                )
            ])
        self.fields['publisher_select'].choices = tuple(
            [(u'', 'Keine Auswahl')] + list(Publisher.objects.all().values_list('code', 'name', named=False)))

    def clean(self):
        cleaned_data = super().clean()

        pub = cleaned_data['publisher']
        pub_sel = cleaned_data['publisher_select']

        if pub and pub_sel:
            msg = 'Bitte wählen sie entweder einen Verlag aus ODER tragen sie einen neuen in das andere Feld ein!'
            self.add_error('publisher', msg)
            self.add_error('publisher_select', msg)
            raise ValidationError(msg)


class ContactForm(forms.ModelForm):
    class Meta:
        model = ContactData
        fields = ['subject', 'text']
        labels = {
            'subject': 'Betreff',
            'text': 'Mitteilung',
        }


class GuestContactForm(forms.ModelForm):
    class Meta:
        model = GuestContactData
        fields = ['sendername', 'senderemail', 'sendertelefon', 'subject', 'text']
        labels = {
            'sendername': 'Ihr Name',
            'senderemail': 'Ihre eMail Adresse',
            'sendertelefon': 'Ihre Telefonnummer (optional)',
            'subject': 'Betreff',
            'text': 'Mitteilung',
        }

    captcha = CaptchaField()


class ReportForm(forms.ModelForm):
    class Meta:
        model = ReportData
        fields = ['subject', 'text', 'tenantusergroup', 'eresource', 'attachment']
        labels = {
            'subject': 'Betreff',
            'text': 'Mitteilung',
            'tenantusergroup': 'Betroffene Nutzergruppe',
            'eresource': 'Betroffene Ressource',
            'attachment': 'Anhang'
        }

    def __init__(self, tenant, *args, **kwargs):
        super(ReportForm, self).__init__(*args, **kwargs)
        self.fields['eresource'].queryset = EResource.objects.filter(
            tenantuser__code__in=tenant.tenantusergroup_set.all()).distinct()
        self.fields['tenantusergroup'].queryset = TenantUserGroup.objects.filter(tenant=tenant)


class AcceptUploadForm(forms.Form):
    code = forms.CharField(required=True, label='Code')
    name = forms.CharField(required=True, label='Name')
    publisher = forms.ChoiceField(required=True, label='Verlag')
    description = forms.CharField(required=False, label='Beschreibung')
    dbis = forms.CharField(required=False, label='DBID ID')
    licences = forms.IntegerField(required=False, label='Lizenzen')
    type = forms.ChoiceField(choices=EResource.ResourceType.choices, label='Typ der Ressource')
    tenantusers = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple, label='Nutzergruppen mit Zugriff')
    active = forms.BooleanField(required=False, label='Aktiviert')

    def __init__(self, tenant, *args, **kwargs):
        super(AcceptUploadForm, self).__init__(*args, **kwargs)
        self.fields['tenantusers'].choices = TenantUserGroup.objects.filter(tenant=tenant).values_list('code', 'name')
        self.fields['publisher'].choices = Publisher.objects.all().values_list('code', 'name')
